/**
 * Script to make an div or table cell editable
 * Set class of the element to 'editable' and include this file in the head of the html page
 * 
 * http://iambot.net/projects/editablejs/
 * 
 * Version: 0.3
 * @author Srinath
 * @email srinath@iambot.net
 *
 */

var EDITABLE;

EDITABLE = function() {
	// Declare a few variables
	var _target = null, // Parent node of the element to be made editable
	_child = null, // The element to be made editable
	_flag = false, // Flag to indicate if any element is being edited
	_node = null, // Node to be replaced. Can either be textNode or simpleNode
	_textNode = null, // A input element of type text for use in tables
	_simpleNode = null, // A textarea element for other elements
	_oldValue = null, // To hold the previous value in case we need to restore it
	_value = null,
	_url = false;

	return {
		
		init: function(url) {
			_simpleNode = document.createElement('textarea');
			_simpleNode.className = 'editing';
			_simpleNode.onkeydown = EDITABLE.updateKey;

			_textNode = document.createElement('input');
			_textNode.type = 'text';
			_textNode.className = 'editing';

			document.onclick = EDITABLE.updateClick;
			
			if(document.getElementsByClassName) {
				elements = document.getElementsByClassName('editable');
			}		
			else {
				elements = getElementsByClassName('editable');
			}	
			
			for(var i=0;i<elements.length;i++) {
				if(elements[i].tagName == 'table' || elements[i].tagName == 'TABLE') {
					var cells = elements[i].getElementsByTagName('td');
					for(j=0;j<cells.length;j++) {
						cells[j].className = 'editable';
						cells[j].onclick = EDITABLE.editNode;
						cells[j].onkeydown = EDITABLE.updateKey;
					}
				}
				else {
					elements[i].onclick = EDITABLE.editNode;
					elements[i].onkeydown = EDITABLE.updateKey;
				}
			}
			
			if(url) {
				_url = url;
			}
		},
		
		updateKey: function(e) {
			if(_flag) {
				if(e == null) {
					e = window.event;
				}
				if(window.event) {
					_key = window.event.keyCode;
				}
				else {
					_key = e.which;
				}
				if(_key != 9 && _key != 13) {
					_node.focus();
					return;
				}
				else {
					try {
						EDITABLE.replaceNode();
						_child.innerHTML = _key == 9 ? _oldValue : _node.value;
						return false;
					} catch (ex) {alert(ex);}
				}	
			}
		},
		
		updateClick: function(e) {
			if(_flag) {
				if(e == null) {
					e = window.event;
				}
				_temp = e.target != null ? e.target : e.srcElement;
				if(_temp.className != 'editable') {
					try {
						EDITABLE.replaceNode();
					} catch(ex) {alert(ex);}
				}
			}
		},
		
		replaceNode: function() {
			_child.removeChild(_node);
			_child.className = 'editable';
			_child.innerHTML = _oldValue;
			_flag = false;
			if(_url) {	
				EDITABLE.updateDB(_node);
			}
		},
		
		updateDB: function(node) {
			if (window.XMLHttpRequest) {
				xmlhttp = new XMLHttpRequest();
			}
			else {
				xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
			}
			xmlhttp.open("GET",_url+"?id="+node.id+"&value="+node.value,true);
			xmlhttp.setRequestHeader('User-Agent','XMLHTTP/1.0');
			xmlhttp.onreadystatechange = function() {
				if (xmlhttp.readyState != 4) {
					return;
				}
				if (xmlhttp.status != 200 && xmlhttp.status != 304) {
					alert('HTTP error ' + xmlhttp.status);
					return;
				}
			};
			if (xmlhttp.readyState == 4) {
				return;
			}
			xmlhttp.send();
		},
		
		editNode: function(e) {
			if(!_flag) {
				if(e == null) {
					e = window.event;
				}
				_child = e.target != null ? e.target : e.srcElement;
				if(_child.className == 'editable') {
					if(e.preventDefault) { 
						e.preventDefault();
						e.stopPropagation();
					}
					else { 
						e.cancelBubble = true;
						e.returnValue = false;
					}
					if(_child.tagName == 'TD' || _child.tagName == 'td') {
						_node = _textNode;
					}
					else {
						_node = _simpleNode;
						_node.style.border = 0;
						_node.style.padding = 0;
					}
					_target = _child.parentNode;
					_node.id = _child.id;
					_node.style.width = _child.offsetWidth+'px';
					_node.style.height = _child.offsetHeight+'px';
					
					if(_target) {
						var value = _child.innerHTML.replace(/\t/gi,'');
						_node.value = value.replace(/^\s*/gi,'');
						_child.className = 'editable editing';
						_oldValue = _child.innerHTML;
						_child.innerHTML = '';
						try {
							_child.appendChild(_node);
							_flag = true;
							_node.focus();
							val = _node.value;
							_node.value = '';
							_node.value = val;
						} catch (exp) {alert(exp);}
					}
				}
			}
			else {
				try {
					EDITABLE.replaceNode();
				} catch(ex) {alert(ex);}
			}
		}
	};
}();